﻿//
// PipedOutputSample.cs
// This file is part of Ghostscript.NET.Samples project
//
// Author: Josip Habjan (habjan@gmail.com, http://www.linkedin.com/in/habjan) 
// Copyright (c) 2013 Josip Habjan. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.IO;
using System.IO.Pipes;
using System.Threading;
using System.Collections.Generic;

using Ghostscript.NET.Processor;

namespace Ghostscript.NET.Samples
{
    /// <summary>
    /// Sample that demonstrates how to tell Ghostscript to write the output result to 
    /// an anonymous pipe (memory) instead of writing it to the disk.
    /// </summary>
    public class PipedOutputSample : ISample
    {
        public void Start()
        {
            string inputFile = @"E:\gss_test\test_postscript.ps";

            GhostscriptPipedOutput gsPipedOutput = new GhostscriptPipedOutput();

            // pipe handle format: %handle%hexvalue
            string outputPipeHandle = "%handle%" + int.Parse(gsPipedOutput.ClientHandle).ToString("X2");

            using (GhostscriptProcessor processor = new GhostscriptProcessor())
            {
                List<string> switches = new List<string>();
                switches.Add("-empty");
                switches.Add("-dQUIET");
                switches.Add("-dSAFER");
                switches.Add("-dBATCH");
                switches.Add("-dNOPAUSE");
                switches.Add("-dNOPROMPT");
                switches.Add("-sDEVICE=pdfwrite");
                switches.Add("-o" + outputPipeHandle);
                switches.Add("-q");
                switches.Add("-f");
                switches.Add(inputFile);

                try
                {
                    processor.StartProcessing(switches.ToArray(), null);

                    byte[] rawDocumentData = gsPipedOutput.Data;

                    //if (writeToDatabase)
                    //{
                    //    Database.ExecSP("add_document", rawDocumentData);
                    //}
                    //else if (writeToDisk)
                    //{
                    //    File.WriteAllBytes(@"E:\gss_test\output\test_piped_output.pdf", rawDocumentData);
                    //}
                }
                catch (Exception ex)
                {
                    Console.WriteLine(ex.Message);
                }
                finally
                {
                    gsPipedOutput.Dispose();
                    gsPipedOutput = null;
                }
            }
        }

        public class GhostscriptPipedOutput : IDisposable
        {
            #region Private variables

            private bool _disposed = false;
            private AnonymousPipeServerStream _pipe;
            private Thread _thread = null;
            private MemoryStream _data = new MemoryStream();

            #endregion

            #region Constructor

            public GhostscriptPipedOutput()
            {
                // pipe handle for the Ghostscript needs to be inheritable
                _pipe = new AnonymousPipeServerStream(PipeDirection.In, HandleInheritability.Inheritable);
                _thread = new Thread(new System.Threading.ParameterizedThreadStart(ReadGhostscriptPipeOutput));
                _thread.Start();
            }

            #endregion

            #region Destructor

            ~GhostscriptPipedOutput()
            {
                Dispose(false);
            }

            #endregion

            #region Dispose

            #region Dispose

            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }

            #endregion

            #region Dispose - disposing

            protected virtual void Dispose(bool disposing)
            {
                if (!_disposed)
                {
                    if (disposing)
                    {
                        if (_pipe != null)
                        {
                            if (_pipe.IsConnected)
                            {
                                _pipe.DisposeLocalCopyOfClientHandle();
                            }

                            _pipe.Dispose(); _pipe = null;
                        }

                        if (_thread != null)
                        {
                            if (_thread.ThreadState == ThreadState.Running)
                            {
                                _thread.Abort();
                            }

                            _thread = null;
                        }
                    }

                    _disposed = true;
                }
            }

            #endregion

            #endregion

            #region ClientHandle

            public string ClientHandle
            {
                get { return _pipe.GetClientHandleAsString(); }
            }

            #endregion

            #region ReadGhostscriptPipeOutput

            public void ReadGhostscriptPipeOutput(object obj)
            {
                BinaryReader reader = new BinaryReader(_pipe);

                int readCount = 0;

                byte[] buffer = new byte[_pipe.InBufferSize];

                while ((readCount = reader.Read(buffer, 0, buffer.Length)) > 0)
                {
                    _data.Write(buffer, 0, readCount);
                }
            }

            #endregion

            #region Data

            public byte[] Data
            {
                get { return _data.ToArray(); }
            }

            #endregion
        }
    }
}
